@@ -19,16 +19,16 @@ build_time_vars = {'ABIFLAGS': '',
  'BOOTSTRAP_HEADERS': '\\',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'powerpc64-unknown-openbsd7.3',
+ 'BUILD_GNU_TYPE': 'aarch64-unknown-openbsd7.3',
  'BYTESTR_DEPS': '\\',
  'CC': 'cc -pthread',
  'CCSHARED': '-fPIC',
- 'CFLAGS': '-DNDEBUG -O2 -pipe   -fPIC -O2 -pipe -O2 -pipe',
+ 'CFLAGS': '-DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe -g -O2 -pipe -g',
  'CFLAGSFORSHARED': '-fPIC',
  'CFLAGS_ALIASING': '',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in '
                 'Makefile.pre.in',
- 'CONFIGURE_CFLAGS': '-O2 -pipe',
+ 'CONFIGURE_CFLAGS': '-O2 -pipe -g',
  'CONFIGURE_CFLAGS_NODIST': '-flto -g -std=c11 '
                             '-Werror=implicit-function-declaration '
                             '-fvisibility=hidden',
@@ -46,7 +46,7 @@ build_time_vars = {'ABIFLAGS': '',
                 "'--sysconfdir=/etc' '--mandir=/usr/local/man' "
                 "'--infodir=/usr/local/info' '--localstatedir=/var' "
                 "'--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' "
-                "'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib/' "
+                "'CFLAGS=-O2 -pipe -g' 'LDFLAGS=-L/usr/local/lib/' "
                 "'CPPFLAGS=-I/usr/local/include' "
                 "'TCLTK_CFLAGS=-I/usr/local/include/tcl8.6 "
                 "-I/usr/local/include/tk8.6 -I/usr/X11R6/include' "
@@ -54,9 +54,9 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFINCLUDEDIR': '/usr/local/include',
  'CONFINCLUDEPY': '/usr/local/include/python3.11',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/usr/obj/ports/Python-3.11.2/Python-3.11.2/coverage.info',
+ 'COVERAGE_INFO': '/build/tmp/pobj/Python-3.11.3/Python-3.11.3/coverage.info',
  'COVERAGE_LCOV_OPTIONS': '--rc lcov_branch_coverage=1',
- 'COVERAGE_REPORT': '/usr/obj/ports/Python-3.11.2/Python-3.11.2/lcov-report',
+ 'COVERAGE_REPORT': '/build/tmp/pobj/Python-3.11.3/Python-3.11.3/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--rc lcov_branch_coverage=1 --branch-coverage '
                             '--title "CPython 3.11 LCOV report [commit $(shell '
                             ')]"',
@@ -83,8 +83,8 @@ build_time_vars = {'ABIFLAGS': '',
  'DLINCLDIR': '.',
  'DLLLIBRARY': '',
  'DOUBLE_IS_ARM_MIXED_ENDIAN_IEEE754': 0,
- 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 1,
- 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 0,
+ 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 0,
+ 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 1,
  'DTRACE': '',
  'DTRACE_DEPS': '\\',
  'DTRACE_HEADERS': '',
@@ -102,7 +102,7 @@ build_time_vars = {'ABIFLAGS': '',
  'EXTRA_CFLAGS': '',
  'EXT_SUFFIX': '.cpython-311.so',
  'FILEMODE': 644,
- 'FLOAT_WORDS_BIGENDIAN': 1,
+ 'FLOAT_WORDS_BIGENDIAN': 0,
  'FREEZE_MODULE': './_bootstrap_python ./Programs/_freeze_module.py',
  'FREEZE_MODULE_BOOTSTRAP': './Programs/_freeze_module',
  'FREEZE_MODULE_BOOTSTRAP_DEPS': 'Programs/_freeze_module',
@@ -593,17 +593,17 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_ZLIB_H': 0,
  'HAVE__GETPTY': 0,
  'HOSTRUNNER': '',
- 'HOST_GNU_TYPE': 'powerpc64-unknown-openbsd7.3',
+ 'HOST_GNU_TYPE': 'aarch64-unknown-openbsd7.3',
  'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include '
                    '/usr/local/include/python3.11 '
                    '/usr/local/include/python3.11',
  'INCLUDEDIR': '/usr/local/include',
  'INCLUDEPY': '/usr/local/include/python3.11',
- 'INSTALL': '/usr/obj/ports/Python-3.11.2/bin/install -c',
- 'INSTALL_DATA': '/usr/obj/ports/Python-3.11.2/bin/install -c -m 644',
- 'INSTALL_PROGRAM': '/usr/obj/ports/Python-3.11.2/bin/install -c -s -m 755',
- 'INSTALL_SCRIPT': '/usr/obj/ports/Python-3.11.2/bin/install -c -m 755',
- 'INSTALL_SHARED': '/usr/obj/ports/Python-3.11.2/bin/install -c -m 755',
+ 'INSTALL': '/build/tmp/pobj//Python-3.11.3/bin/install -c',
+ 'INSTALL_DATA': '/build/tmp/pobj//Python-3.11.3/bin/install -c -m 644',
+ 'INSTALL_PROGRAM': '/build/tmp/pobj//Python-3.11.3/bin/install -c  -m 755',
+ 'INSTALL_SCRIPT': '/build/tmp/pobj//Python-3.11.3/bin/install -c -m 755',
+ 'INSTALL_SHARED': '/build/tmp/pobj//Python-3.11.3/bin/install -c -m 755',
  'INSTSONAME': 'libpython3.11.so.0.0',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
@@ -617,8 +617,8 @@ build_time_vars = {'ABIFLAGS': '',
  'LIBDEST': '/usr/local/lib/python3.11',
  'LIBDIR': '/usr/local/lib',
  'LIBEXPAT_A': 'Modules/expat/libexpat.a',
- 'LIBEXPAT_CFLAGS': '-DNDEBUG -O2 -pipe   -fPIC -O2 -pipe -O2 -pipe -flto -g '
-                    '-std=c11 -Werror=implicit-function-declaration '
+ 'LIBEXPAT_CFLAGS': '-DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe -g -O2 -pipe -g '
+                    '-flto -g -std=c11 -Werror=implicit-function-declaration '
                     '-fvisibility=hidden -I/usr/local/include '
                     '-fprofile-instr-use=code.profclangd -I./Include/internal '
                     '-I. -I./Include -I/usr/local/include -fPIC -fPIC',
@@ -628,8 +628,8 @@ build_time_vars = {'ABIFLAGS': '',
  'LIBM': '-lm',
  'LIBMPDEC_A': 'Modules/_decimal/libmpdec/libmpdec.a',
  'LIBMPDEC_CFLAGS': '-I./Modules/_decimal/libmpdec -DCONFIG_64=1 -DANSI=1 '
-                    '-DHAVE_UINT128_T=1 -DNDEBUG -O2 -pipe   -fPIC -O2 -pipe '
-                    '-O2 -pipe -flto -g -std=c11 '
+                    '-DHAVE_UINT128_T=1 -DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe '
+                    '-g -O2 -pipe -g -flto -g -std=c11 '
                     '-Werror=implicit-function-declaration -fvisibility=hidden '
                     '-I/usr/local/include -fprofile-instr-use=code.profclangd '
                     '-I./Include/internal -I. -I./Include -I/usr/local/include '
@@ -888,7 +888,7 @@ build_time_vars = {'ABIFLAGS': '',
  'OPENSSL_LDFLAGS': '',
  'OPENSSL_LIBS': '-lssl -lcrypto',
  'OPENSSL_RPATH': '',
- 'OPT': '-DNDEBUG -O2 -pipe   -fPIC',
+ 'OPT': '-DNDEBUG -O2 -pipe -g  -fPIC',
  'OTHER_LIBTOOL_OPT': '',
  'PACKAGE_BUGREPORT': 0,
  'PACKAGE_NAME': 0,
@@ -924,42 +924,43 @@ build_time_vars = {'ABIFLAGS': '',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
  'PY_BUILTIN_HASHLIB_HASHES': '"md5,sha1,sha256,sha512,sha3,blake2"',
- 'PY_BUILTIN_MODULE_CFLAGS': '-DNDEBUG -O2 -pipe   -fPIC -O2 -pipe -O2 -pipe '
-                             '-flto -g -std=c11 '
+ 'PY_BUILTIN_MODULE_CFLAGS': '-DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe -g -O2 '
+                             '-pipe -g -flto -g -std=c11 '
                              '-Werror=implicit-function-declaration '
                              '-fvisibility=hidden -I/usr/local/include '
                              '-fprofile-instr-use=code.profclangd '
                              '-I./Include/internal -I. -I./Include '
                              '-I/usr/local/include -fPIC '
                              '-DPy_BUILD_CORE_BUILTIN',
- 'PY_CFLAGS': '-DNDEBUG -O2 -pipe   -fPIC -O2 -pipe -O2 -pipe',
+ 'PY_CFLAGS': '-DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe -g -O2 -pipe -g',
  'PY_CFLAGS_NODIST': '-flto -g -std=c11 -Werror=implicit-function-declaration '
                      '-fvisibility=hidden -I/usr/local/include '
                      '-fprofile-instr-use=code.profclangd -I./Include/internal',
  'PY_COERCE_C_LOCALE': 1,
- 'PY_CORE_CFLAGS': '-DNDEBUG -O2 -pipe   -fPIC -O2 -pipe -O2 -pipe -flto -g '
-                   '-std=c11 -Werror=implicit-function-declaration '
+ 'PY_CORE_CFLAGS': '-DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe -g -O2 -pipe -g '
+                   '-flto -g -std=c11 -Werror=implicit-function-declaration '
                    '-fvisibility=hidden -I/usr/local/include '
                    '-fprofile-instr-use=code.profclangd -I./Include/internal '
                    '-I. -I./Include -I/usr/local/include -fPIC -DPy_BUILD_CORE',
  'PY_CORE_LDFLAGS': '-L/usr/local/lib/ -flto -g '
-                    '-L/usr/obj/ports/Python-3.11.2/Python-3.11.2 '
+                    '-L/build/tmp/pobj//Python-3.11.3/Python-3.11.3 '
                     '-L/usr/local/lib/',
  'PY_CPPFLAGS': '-I. -I./Include -I/usr/local/include',
  'PY_ENABLE_SHARED': 1,
  'PY_FORMAT_SIZE_T': '"z"',
  'PY_LDFLAGS': '-L/usr/local/lib/',
- 'PY_LDFLAGS_NODIST': '-flto -g -L/usr/obj/ports/Python-3.11.2/Python-3.11.2 '
+ 'PY_LDFLAGS_NODIST': '-flto -g -L/build/tmp/pobj//Python-3.11.3/Python-3.11.3 '
                       '-L/usr/local/lib/',
  'PY_LDFLAGS_NOLTO': '-L/usr/local/lib/ -flto=thin '
-                     '-L/usr/obj/ports/Python-3.11.2/Python-3.11.2 '
+                     '-L/build/tmp/pobj//Python-3.11.3/Python-3.11.3 '
                      '-L/usr/local/lib/',
  'PY_SQLITE_ENABLE_LOAD_EXTENSION': 1,
  'PY_SQLITE_HAVE_SERIALIZE': 1,
  'PY_SSL_DEFAULT_CIPHERS': 1,
  'PY_SSL_DEFAULT_CIPHER_STRING': 0,
- 'PY_STDMODULE_CFLAGS': '-DNDEBUG -O2 -pipe   -fPIC -O2 -pipe -O2 -pipe -flto '
-                        '-g -std=c11 -Werror=implicit-function-declaration '
+ 'PY_STDMODULE_CFLAGS': '-DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe -g -O2 -pipe '
+                        '-g -flto -g -std=c11 '
+                        '-Werror=implicit-function-declaration '
                         '-fvisibility=hidden -I/usr/local/include '
                         '-fprofile-instr-use=code.profclangd '
                         '-I./Include/internal -I. -I./Include '
@@ -974,7 +975,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.11.2/Python-3.11.2',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/build/tmp/pobj/Python-3.11.3/Python-3.11.3',
  'SCRIPTDIR': '/usr/local/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SHELL': '/bin/sh',
@@ -987,7 +988,7 @@ build_time_vars = {'ABIFLAGS': '',
  'SIZEOF_FPOS_T': 8,
  'SIZEOF_INT': 4,
  'SIZEOF_LONG': 8,
- 'SIZEOF_LONG_DOUBLE': 8,
+ 'SIZEOF_LONG_DOUBLE': 16,
  'SIZEOF_LONG_LONG': 8,
  'SIZEOF_OFF_T': 8,
  'SIZEOF_PID_T': 4,
@@ -1018,10 +1019,10 @@ build_time_vars = {'ABIFLAGS': '',
  'SYS_SELECT_WITH_SYS_TIME': 1,
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.11.2/Python-3.11.2 '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/tmp/pobj/Python-3.11.3/Python-3.11.3 '
                './python -E',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.11.2/Python-3.11.2 '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/tmp/pobj/Python-3.11.3/Python-3.11.3 '
                './python -E ./Tools/scripts/run_tests.py',
  'TESTSUBDIRS': 'ctypes/test \\',
  'TESTTIMEOUT': 1200,
@@ -1052,8 +1053,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/usr/obj/ports/Python-3.11.2/Python-3.11.2',
- 'abs_srcdir': '/usr/obj/ports/Python-3.11.2/Python-3.11.2',
+ 'abs_builddir': '/build/tmp/pobj/Python-3.11.3/Python-3.11.3',
+ 'abs_srcdir': '/build/tmp/pobj/Python-3.11.3/Python-3.11.3',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
  'prefix': '/usr/local',
